@@ -1,3 +1,4 @@
+from typing import Union
 import os
 from numbers import Number
 import h5py
@@ -5,15 +6,18 @@ import numpy as np
 import nrrd
 from numpy.lib.mixins import NDArrayOperatorsMixin
 
+from scipy.ndimage import gaussian_filter
+
 import tifffile
 import cc3d
-from cloudvolume.lib import Bbox, yellow
+from cloudvolume.lib import yellow, Bbox
+
+from chunkflow.lib.bounding_boxes import BoundingBox
+
 # from typing import Tuple
 # Offset = Tuple[int, int, int]
 from .validate import validate_by_template_matching
 
-# from memory_profiler import profile
-
 
 class Chunk(NDArrayOperatorsMixin):
     r"""
@@ -58,7 +62,7 @@ class Chunk(NDArrayOperatorsMixin):
     _HANDLED_TYPES = (np.ndarray, Number)
     
     @classmethod
-    def from_array(cls, array: np.ndarray, bbox: Bbox, voxel_size: tuple = None):
+    def from_array(cls, array: np.ndarray, bbox: BoundingBox, voxel_size: tuple = None):
         """
         :param array: ndarray data
         :param bbox: cloudvolume bounding box
@@ -68,7 +72,7 @@ class Chunk(NDArrayOperatorsMixin):
         return cls(array, voxel_offset=bbox.minpt, voxel_size=voxel_size)
     
     @classmethod
-    def from_bbox(cls, bbox: Bbox, dtype: type = np.uint8,
+    def from_bbox(cls, bbox: BoundingBox, dtype: type = np.uint8,
             voxel_size: tuple=None, all_zero: bool=False):
         """
         :param bbox: bounding box of chunk.
@@ -76,7 +80,7 @@ class Chunk(NDArrayOperatorsMixin):
         :param all_zero: is it all zero or with a fancy function
         :return: a new chunk
         """
-        assert isinstance(bbox, Bbox)
+        assert isinstance(bbox, BoundingBox)
         size = bbox.maxpt - bbox.minpt
         return cls.create(size=size, dtype=dtype, voxel_offset=bbox.minpt,
             voxel_size=voxel_size, all_zero=all_zero)
@@ -118,6 +122,10 @@ class Chunk(NDArrayOperatorsMixin):
 
         return cls(chunk, voxel_offset=voxel_offset, voxel_size=voxel_size)
 
+    def clone(self):
+        return Chunk(self.array.copy(), 
+            voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
+
     @classmethod
     def from_nrrd(cls, file_name: str, voxel_offset: tuple=None, dtype: str = None,
             voxel_size: tuple=None):
@@ -130,17 +138,11 @@ class Chunk(NDArrayOperatorsMixin):
     def to_nrrd(self, file_name: str=None):
         if file_name is None:
             file_name = f'{self.bbox.to_filename()}.nrrd'
-        print('write chunk to file: ', file_name)
+        elif not file_name.endswith('.nrrd'):
+            file_name += f'_{self.bbox.to_filename()}.nrrd'
 
-        if self.array.dtype==np.float32:
-            # visualization in float32 is not working correctly in ImageJ
-            # this might not work correctly if you want to save the image as it is!
-            print(yellow('transforming data type from float32 to uint8'))
-            img = self.array*255 
-            img = img.astype( np.uint8 )
-        else:
-            img = self.array
-        nrrd.write(file_name, img)
+        print('write chunk to file: ', file_name)
+        nrrd.write(file_name, self.array)
 
     @classmethod
     def from_tif(cls, file_name: str, voxel_offset: tuple=None, dtype: str = None,
@@ -149,6 +151,7 @@ class Chunk(NDArrayOperatorsMixin):
 
         if dtype:
             arr = arr.astype(dtype)
+        print(f'read tif chunk with size of {arr.shape}, voxel offset: {voxel_offset}, voxel size: {voxel_size}')
         return cls(arr, voxel_offset=voxel_offset, voxel_size=voxel_size)
     
     def to_tif(self, file_name: str=None):
@@ -173,16 +176,27 @@ class Chunk(NDArrayOperatorsMixin):
                 voxel_size: tuple = None,
                 cutout_start: tuple = None,
                 cutout_stop: tuple = None,
-                cutout_size: tuple = None):
+                cutout_size: tuple = None,
+                zero_filling: bool = False,
+                dtype: str = None):
+
+        assert os.path.exists(file_name)
+        
         if cutout_start is not None and cutout_size is not None:
             cutout_stop = tuple(t+s for t, s in zip(cutout_start, cutout_size))
 
         if not h5py.is_hdf5(file_name):
             assert cutout_start is not None 
             assert cutout_stop is not None
-            bbox = Bbox.from_list([*cutout_start, *cutout_stop])
+            bbox = BoundingBox.from_list([*cutout_start, *cutout_stop])
             file_name += f'{bbox.to_filename()}.h5'
 
+            if not os.path.exists(file_name) and zero_filling:
+                # fill with zero
+                assert dtype is not None
+                print(f'file do not exist, will fill with zero: {file_name}')
+                return cls.from_bbox(bbox, dtype=dtype, voxel_size=voxel_size, all_zero=True)
+
         with h5py.File(file_name, 'r') as f:
             if dataset_path is None:
                 for key in f.keys():
@@ -236,7 +250,9 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
 
     def to_h5(self, file_name: str, with_offset: bool=True, 
                 chunk_size: tuple=(64,64,64),
-                with_unique: bool= True, compression="gzip"):
+                with_unique: bool= True, 
+                compression="gzip",
+                voxel_size: tuple = None):
         """
         :param file_name: output file name. If it is not end with h5, the coordinate will be appended to the file name.
         :param with_offset: save the voxel offset or not
@@ -257,8 +273,11 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
 
         with h5py.File(file_name, 'w') as f:
             f.create_dataset('/main', data=self.array, chunks=chunk_size, compression=compression)
-            if self.voxel_size:
-                f.create_dataset('/voxel_size', data=self.voxel_size)
+            if voxel_size is None and self.voxel_size is not None:
+                voxel_size = self.voxel_size
+            if voxel_size is not None:
+                f.create_dataset('/voxel_size', data=voxel_size)
+
             if with_offset and self.voxel_offset is not None:
                 f.create_dataset('/voxel_offset', data=self.voxel_offset)
 
@@ -267,6 +286,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
                 if unique[0]:
                     unique = unique[1:]
                 f.create_dataset('/unique_nonzeros', data = unique)
+        return file_name
 
     def __array__(self):
         return self.array
@@ -296,7 +316,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         
         if type(result) is tuple:
             # multiple return values
-            return tuple(type(self)(x, voxel_offset=self.voxel_offset) for x in result)
+            return tuple(type(self)(x, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size) for x in result)
         elif method == 'at':
             # no return value
             return None
@@ -304,7 +324,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
             return result
         elif isinstance(result, np.ndarray):
             # one return value
-            return type(self)(result, voxel_offset=self.voxel_offset)
+            return type(self)(result, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
         else:
             return result
 
@@ -315,7 +335,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         self.array[key] = value
 
     def __repr__(self):
-        return f'array: {self.array}\n global offset: {self.voxel_offset}'
+        return f'array: {self.array}\n voxel offset: {self.voxel_offset} \n voxel size: {self.voxel_size}'
     
     def __eq__(self, value):
         if isinstance(value, type(self)):
@@ -343,7 +363,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         if self.voxel_size is not None or self.voxel_size != (1, 1, 1):
             props['voxel_size'] = self.voxel_size
         return props 
-
+    
     @property
     def slices(self) -> tuple:
         """
@@ -363,6 +383,10 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
 
     @property
     def is_affinity_map(self) -> bool:
+        return self.array.ndim == 4 and self.shape[0] == 3 and self.array.dtype == np.float32
+    
+    @property
+    def is_probability_map(self) -> bool:
         return self.array.ndim == 4 and self.array.dtype == np.float32
 
     @property
@@ -376,11 +400,12 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
             return self.voxel_offset
 
     @property
-    def bbox(self) -> Bbox:
+    def bbox(self) -> BoundingBox:
         """
-        :getter: the cloudvolume bounding box in the big volume
+        :getter: the bounding box in the big volume
         """
-        return Bbox.from_delta(self.voxel_offset, self.array.shape[-3:])
+        bbox = BoundingBox.from_delta(self.voxel_offset, self.array.shape[-3:])
+        return bbox
     
     @property
     def ndim(self) -> int:
@@ -401,14 +426,14 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
     def astype(self, dtype: np.dtype):
         if dtype != self.array.dtype:
             new_array = self.array.astype(dtype)
-            return type(self)(new_array, voxel_offset=self.voxel_offset)
         else:
-            return self
+            new_array = self.array
+        return Chunk(new_array, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
 
     def ascontiguousarray(self):
-        np.ascontiguousarray(self.array)
-        return self
-    
+        new_array = np.ascontiguousarray(self.array)
+        return Chunk(new_array, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
+
     def max(self, *args, **kwargs):
         return self.array.max(*args, **kwargs)
 
@@ -418,20 +443,26 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
     def transpose(self):
         """To-Do: support arbitrary axis transpose"""
         new_array = self.array.transpose()
-        new_voxel_offset = self.voxel_offset[::-1]
-        return type(self)(new_array, voxel_offset=new_voxel_offset)
-    
+        if self.voxel_offset is not None:
+            voxel_offset = self.voxel_offset[::-1]
+        else:
+            voxel_offset = self.voxel_offset
+        if self.voxel_size is not None:
+            voxel_size = self.voxel_size[::-1]
+        else:
+            voxel_size = self.voxel_size
+        return Chunk(new_array, voxel_offset=voxel_offset, voxel_size=voxel_size)
+
+
     def fill(self, x):
         self.array.fill(x)
 
     def squeeze_channel(self) -> np.ndarray:
         """given a 4D array, squeeze the channel axis."""
         assert self.array.ndim == 4
-        axis = 0
-        arr = np.squeeze(self, axis=0)
-        voxel_offset = self.voxel_offset[:axis] + self.voxel_offset[axis+1:]
-        return Chunk(arr, voxel_offset=voxel_offset, voxel_size=self.voxel_size)
-    
+        new_array = np.squeeze(self, axis=0)
+        return Chunk(new_array, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
+
     # @profile(precision=0)
     def channel_voting(self):
         assert self.ndim == 4
@@ -448,7 +479,7 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         ret *= mask
         return Chunk(ret, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
 
-    def crop_margin(self, margin_size: tuple = None, output_bbox: Bbox=None):
+    def crop_margin(self, margin_size: tuple = None, output_bbox: BoundingBox=None):
 
         if margin_size:
             if len(margin_size) == 3 and self.ndim == 4:
@@ -595,3 +626,11 @@ ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         """
         validate_by_template_matching(self.array)
 
+    def gaussian_filter_2d(self, sigma: Union[int, tuple, list] = 1):
+        """gaussion filter for smoothing or blurring
+
+        Args:
+            sigma (Union[int, tuple, list]): the standard deviation of gaussian filter
+        """
+        for z in range(self.shape[-3]):
+            self.array[..., z, :, :] = gaussian_filter(self.array[..., z, :, :], sigma)
