@@ -1,3 +1,4 @@
+import logging
 from chunkflow.lib.aws.cloud_watch import CloudWatch
 
 from .base import OperatorBase
@@ -14,14 +15,12 @@ class CloudWatchOperator(OperatorBase):
     """
     def __init__(self,
                  log_name: str = 'chunkflow',
-                 name: str = 'cloud-watch',
-                 verbose: bool = True):
-        super().__init__(name=name, verbose=verbose)
+                 name: str = 'cloud-watch'):
+        super().__init__(name=name)
 
         # write to aws cloud watch
         self.aws_cloud_watch = CloudWatch(log_name)
 
     def __call__(self, log):
         self.aws_cloud_watch.put_metric_data(log)
-        if self.verbose:
-            print('log submitted to cloud watch: ', log)
+        logging.info(f'log submitted to cloud watch: {log}')
