@@ -13,13 +13,12 @@ class NormalizeSectionShangOperator(OperatorBase):
                  nominalmin: float = None,
                  nominalmax: float = None,
                  clipvalues: bool = False,
-                 name: str = 'normalize-gray',
-                 verbose=False):
+                 name: str = 'normalize-gray'):
         """
         nominalmin/max: (float)
         clipvalues: (bool)
         """
-        super().__init__(name=name, verbose=verbose)
+        super().__init__(name=name)
         assert nominalmin < nominalmax
 
         self.nominalmin = nominalmin
